package de.hdm_stuttgart.battlearena.Controller;

import de.hdm_stuttgart.battlearena.Controller.Enum.GameState;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.ENUMs.AccountType;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.Exceptions.DatabaseException;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.Persistence;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.RuntimeInfo;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.ThreadStartup1;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.ThreadStartup2;
import javafx.application.Platform;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.geometry.Rectangle2D;
import javafx.scene.control.Label;
import javafx.scene.image.ImageView;
import javafx.scene.layout.BorderPane;
import javafx.stage.Screen;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.IOException;
import java.net.URL;
import java.util.*;

public class LoadingScreenController implements Initializable {
    @FXML
    private Label tips;
    @FXML
    private BorderPane parent;
    @FXML
    private ImageView logo, throbber;

    private int counter = 0;
    private boolean statsLoaded = false;
    private final String[] loadingTips = {
            "Did you know, that since 2009, Martin isn't on the FBI list?",
            "BattleArena is actually derived from the combination of the words 'Battle' and 'Arena'.",
            "Yannik ist over 1,80m tall.",
            "Electricity is a vital part to play this game.",
            "There is a typo in one of the tips.",
            "Over 1000 people play games every day.",
            "A lot of people believe Nikola Tesla was the first to invent electricity but some clouds beat him to it.",
            "Monkeys usually don't have keys.",
            "While you're reading this, you've wasted 2 seconds.",
            "If you die, you're dead.",
            "If you take damage, you loose hp.",
            "Did you know that Java is also an Island?",
            "This loading screen might never end.",
            "Get the BattleArena Ultimate-Edition for only 420,69€.",
            "To move left, press one of the keys on your keyboard.",
            "To use the gravity-gun, close the game and start Half Life 2.",
            "Buying this game was a good decision! (no refunds).",
            "Apparently Jonas is above the law.",
            "Jonas has lost his Game Dev Role in Discord.",
            "Maxe is a light mode user. Poor soul...",
            "Did you know, that Elias almost destroyed our GitLab Repo?"
    };

    private final List<String> shuffledTips = Arrays.asList(loadingTips);
    private static final Logger log = LogManager.getLogger(Persistence.class);
    Rectangle2D screen = Screen.getPrimary().getVisualBounds();
    ThreadStartup1 thread1 = new ThreadStartup1();
    ThreadStartup2 thread2 = new ThreadStartup2();
    Persistence persistence = Persistence.getInstance();

    Thread tipsThread = new Thread(() -> {
        try {
            setLoadingTips();
        } catch (InterruptedException e) {
            log.info("Data finished loading");
        }
    });

    Thread loadStatistics = new Thread(() -> {
        try {
            Thread.sleep(4000);
            loadStatistics();
        } catch (IOException e) {
            throw new RuntimeException();
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }
    });

    Thread waitingForThread1 = new Thread(() -> {
        try {
            thread1.join();
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }
        thread2.start();
    });

    @Override
    public void initialize(URL url, ResourceBundle resourceBundle) {
        Collections.shuffle(shuffledTips);
        setImages();

        tipsThread.start();
        thread1.start();
        waitingForThread1.start(); // basically thread1.join() but making it in separate thread otherwise it will block JavaFX thread
        loadStatistics.start();
    }

    private void setImages() {
        logo.setTranslateY(screen.getHeight() * 0.08);
        System.out.println(screen.getHeight());
        logo.setFitWidth(screen.getWidth() * 0.3);
        throbber.setFitWidth(screen.getWidth() * 0.07);
        logo.setPreserveRatio(true);
        throbber.setPreserveRatio(true);
    }

    private void setLoadingTips() throws InterruptedException {
        while (!statsLoaded && counter < loadingTips.length) {
            String nextTip = shuffledTips.get(counter);
            Platform.runLater(() -> tips.setText(nextTip)); // JavaFX method to queue a task that needs to be executed on the JavaFX application thread (update ui components)
            counter++;

//            check if statsLoaded is still false before sleeping
            if (!statsLoaded) {
                if (nextTip.length() > 60) {
                    Thread.sleep(6000);
                } else {
                    Thread.sleep(3500);
                }
                setLoadingTips();
            }
        }
        if (counter == loadingTips.length) {
            Collections.shuffle(shuffledTips);
            counter = 0;
            setLoadingTips();
        }
    }

    private void loadStatistics() throws IOException {
        if (persistence.getAccount().getAccountType() != AccountType.NONE) {
            try {
                persistence.loadPlayerStatistics();
                statsLoaded = true;
            } catch (DatabaseException e) {
                log.error(e);
            }
        }

        Platform.runLater(() -> {
            try {
                tipsThread.interrupt();
                RuntimeInfo.getInstance().setGameState(GameState.MENU);
                parent.getScene().setRoot(FXMLLoader.load(Objects.requireNonNull(getClass().getResource("/fxml/MenuBorderPane.fxml"))));
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        });
    }
}
